using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace ColorSpace
{
	delegate void RotateColorDelegate();

	/// <summary>
	/// Summary description for ColorForm.
	/// </summary>
	public class ColorBrushForm : System.Windows.Forms.Form
	{
		RotateColorDelegate rotateColor;
		StringFormat format;
			
		private System.Windows.Forms.Button buttonStart;
		private System.Windows.Forms.Button buttonStop;
		private System.Windows.Forms.StatusBar statusBar1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public ColorBrushForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			rotateColor = new RotateColorDelegate(RotateColor);
			buttonStart.Enabled = true;
			buttonStop.Enabled = false;
			MinimumSize = new Size(700,700);
			format = new StringFormat();
			format.Alignment = StringAlignment.Center;
			
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.buttonStart = new System.Windows.Forms.Button();
			this.buttonStop = new System.Windows.Forms.Button();
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.SuspendLayout();
			// 
			// buttonStart
			// 
			this.buttonStart.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.buttonStart.Location = new System.Drawing.Point(72, 216);
			this.buttonStart.Name = "buttonStart";
			this.buttonStart.Size = new System.Drawing.Size(96, 24);
			this.buttonStart.TabIndex = 0;
			this.buttonStart.Text = "Change Colors";
			this.buttonStart.Click += new System.EventHandler(this.buttonStart_Click);
			// 
			// buttonStop
			// 
			this.buttonStop.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.buttonStop.Location = new System.Drawing.Point(184, 216);
			this.buttonStop.Name = "buttonStop";
			this.buttonStop.Size = new System.Drawing.Size(96, 24);
			this.buttonStop.TabIndex = 1;
			this.buttonStop.Text = "Stop Thread";
			this.buttonStop.Click += new System.EventHandler(this.buttonStop_Click);
			// 
			// statusBar1
			// 
			this.statusBar1.Location = new System.Drawing.Point(0, 244);
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.Size = new System.Drawing.Size(292, 22);
			this.statusBar1.TabIndex = 2;
			// 
			// ColorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 266);
			this.Controls.Add(this.statusBar1);
			this.Controls.Add(this.buttonStop);
			this.Controls.Add(this.buttonStart);
			this.Name = "ColorForm";
			this.Text = "ColorForm";
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.ColorForm_Paint);
			this.ResumeLayout(false);

		}
		#endregion

		private void ColorForm_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			int count = 0;
			Graphics g = e.Graphics;
			String[] allNames = Enum.GetNames(typeof(System.Drawing.KnownColor));
			Rectangle rectBox = new Rectangle(0, 0, this.ClientRectangle.Width/6, 
				(this.ClientRectangle.Height - 30)/31);
			
			DrawCenterText(g, ref rectBox, "UI Colors");
			foreach (String colorName in allNames)
			{
				KnownColor known = (KnownColor) Enum.Parse(typeof(System.Drawing.KnownColor), colorName);
				Color colorCurrent = Color.FromKnownColor(known);
				using (Brush brush = new SolidBrush(colorCurrent)) 
				{
					g.FillRectangle(brush, rectBox);
					g.DrawString(colorName, this.Font, GetTextBrush(colorCurrent), rectBox);
					rectBox.X += rectBox.Width;
					TestEndUIColors(g, ref rectBox, count);
					TestNewLine(ref rectBox);
				}
				count++;
			}														  
		}

		public void DrawCenterText(Graphics g, ref Rectangle rect, String text) 
		{
			Rectangle rectWide = new Rectangle(rect.X, rect.Y, this.ClientRectangle.Width, rect.Height);
			g.DrawString(text, this.Font, GetTextBrush(this.BackColor), rectWide, this.format);
			StartNewLine(ref rect);
		}

		public void TestEndUIColors(Graphics g, ref Rectangle rect, int count) 
		{
			if (count == 26) 
			{
				StartNewLine(ref rect);
				DrawCenterText(g, ref rect, "Known Colors");
			}
		}

		public void TestNewLine(ref Rectangle rect) 
		{
			if (rect.X > this.ClientRectangle.Width - rect.Width) 
			{
				StartNewLine(ref rect);
			}		
		}

		public void StartNewLine(ref Rectangle rect) 
		{
			rect.X = 0;
			rect.Y += rect.Height;
		}

		public Brush GetTextBrush(Color color) 
		{
			if (color.GetBrightness() > 0.4) 
			{
				return Brushes.Black;
			}
			return Brushes.White;
			
		}

		private bool bCancel;
		private void RotateColor() 
		{
			int count = 0;
			String[] allNames = Enum.GetNames(typeof(System.Drawing.KnownColor));
			foreach (String colorName in allNames)
			{
				if (!colorName.ToLower().Equals("transparent")) 
				{
					KnownColor known = (KnownColor) Enum.Parse(typeof(System.Drawing.KnownColor), colorName);
					BackColor = Color.FromKnownColor(known);
					statusBar1.Text = 
						String.Format("{0} ({1} of {2})", colorName, count, allNames.Length);
					Refresh();
					System.Threading.Thread.Sleep(500);
				}
				count++;
				if (bCancel) break;
			}
		}

		private void buttonStart_Click(object sender, System.EventArgs e)
		{
			buttonStart.Enabled = false;
			buttonStop.Enabled = true;
			bCancel = false;
			rotateColor.BeginInvoke(null, null);
		}

		object stateLock = new object();
		private void buttonStop_Click(object sender, System.EventArgs e)
		{
			lock (stateLock) 
			{
				buttonStart.Enabled = true;
				buttonStop.Enabled = false;
				bCancel = true;
			}
		}
	}
}
